/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-1999 Regents of the University of Minnesota
**    Portions Copyright 1999 Sistina Software, Inc.  <kpreslan@sistina.com>
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@lcse.umn.edu>
** 
**    Module name: main.c
**
*******************************************************************************
******************************************************************************/

#define __NO_VERSION__ 
#define EXPORT_SYMTAB
#include <linux/module.h>
#include <linux/version.h>
char kernel_version [] = UTS_RELEASE;

#define EXTERN
#include "gfs.h"
#include "gfs_locking.h"
#include "blklist.h"
#include "bmap.h"
#include "inode.h"
#include "super.h"





struct file_system_type gfs_fs_type =
{
  "gfs",
  FS_REQUIRES_DEV,
  gfs_read_super,
  NULL
};


extern struct inode_operations gfs_blkdev_inode_operations;
extern struct inode_operations gfs_chrdev_inode_operations;

extern semaphore_t gfs_lock_sema;





/******************************************************************************
*******************************************************************************
**
** init_module()
**
** Description:
**   Register GFS as a filesystem witht the rest of the kernel
**
** Returns:
**   0 on success, -EXXXX on failure
**
*******************************************************************************
******************************************************************************/

int init_module(void)
{
  int error;


  /*  Kludge so device node permissions and ownership are immediately changed
      across machines.  */

  gfs_blkdev_inode_operations = blkdev_inode_operations;
  gfs_blkdev_inode_operations.revalidate = gfs_irevalidate;
  gfs_chrdev_inode_operations = chrdev_inode_operations;
  gfs_chrdev_inode_operations.revalidate = gfs_irevalidate;

  GFS_SEMA_INIT(&gfs_lock_sema);


#ifdef DEBUG
  printk(KERN_DEBUG "sizeof(gfs_sb_t) = %d\n", (int)sizeof(gfs_sb_t));
  printk(KERN_DEBUG "sizeof(gfs_rindex_t) = %d\n", (int)sizeof(gfs_rindex_t));
  printk(KERN_DEBUG "sizeof(gfs_rgrp_t) = %d\n", (int)sizeof(gfs_rgrp_t));
  printk(KERN_DEBUG "sizeof(gfs_dinode_t) = %d\n", (int)sizeof(gfs_dinode_t));
  printk(KERN_DEBUG "sizeof(gfs_dirent_t) = %d\n", (int)sizeof(gfs_dirent_t));
  printk(KERN_DEBUG "sizeof(gfs_leaf_t) = %d\n", (int)sizeof(gfs_leaf_t));

  printk(KERN_DEBUG "\n");

  printk(KERN_DEBUG "sizeof(gfs_rgrpd_t) = %d\n", (int)sizeof(gfs_rgrpd_t));
  printk(KERN_DEBUG "sizeof(gfs_glock_t) = %d\n", (int)sizeof(gfs_glock_t));
  printk(KERN_DEBUG "sizeof(gfs_bhdata_t) = %d\n", (int)sizeof(gfs_bhdata_t));
  printk(KERN_DEBUG "sizeof(gfs_diroff_t) = %d\n", (int)sizeof(gfs_diroff_t));
  printk(KERN_DEBUG "sizeof(gfs_inode_t) = %d\n", (int)sizeof(gfs_inode_t));
  printk(KERN_DEBUG "sizeof(gfs_sbd_t) = %d\n", (int)sizeof(gfs_sbd_t));
  printk(KERN_DEBUG "sizeof(gfs_dcached_t) = %d\n", (int)sizeof(gfs_dcached_t));

  printk(KERN_DEBUG "\n");

  printk(KERN_DEBUG "sizeof(gfs_blklist_t) = %d\n", (int)sizeof(gfs_blklist_t));
  printk(KERN_DEBUG "sizeof(gfs_direct_t) = %d\n", (int)sizeof(gfs_direct_t));
#endif  /*  DEBUG  */

  if ((error = register_filesystem(&gfs_fs_type)) < 0)
    return(error);

  printk(KERN_INFO "GFS  v%d.%d.%d (%s release) (%s %s) installed\n",
	 GFSMajorVersion, GFSMinorVersion, GFSUpdateLevel,
	 GFS_RELEASE_NAME,
	 __DATE__, __TIME__);

  return(0);
}


/******************************************************************************
*******************************************************************************
**
** cleanup_module()
**
** Description:
**   Unregister the filesystem
**
*******************************************************************************
******************************************************************************/

void cleanup_module(void)
{
  unregister_filesystem(&gfs_fs_type);

  printk(KERN_INFO "GFS  uninstalled\n");
}



/*  Symbols  */
EXPORT_SYMBOL(register_lock_proto);
EXPORT_SYMBOL(unregister_lock_proto);
