/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-1999 Regents of the University of Minnesota
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Seth Van Oort <vano0023@tc.umn.edu>
** 
**    Module name: gfs_profile.c
**
**    Description:
**	This module contains performance tuning and stat gathering code
**
*******************************************************************************
******************************************************************************/

#include "gfs_profile.h"

#ifdef PROFILE
struct gfs_profile gfs_prof[NR_FUNCTIONS];
struct timeval gfs_tv;
#endif

gfs_tune_t tune = {{40, 90}, };

/******************************************************************************
*******************************************************************************
**
** gfs_init_tunevals
**
** Description:
**   initializes the byte values based on the percentage values. Called upon
**   mounting or by an ioctl that adjusts the values.
**
** Input(s):
**   sbd - the gfs_sbd_t of the superblock
**   allocate - non-zero to allocate a new gfs_tune_t structure (mounting)
**   
**
** Output(s):
**
** Returns:
**   
**
*******************************************************************************
******************************************************************************/


int gfs_init_tunevals(gfs_sbd_t *sbd, int allocate) {
  uint32 blocksize = sbd->sd_sb.sb_bsize;

  if (allocate) {
    sbd->tune = (gfs_tune_t *) gmalloc(sizeof(gfs_tune_t));

    if (!sbd->tune)
      return (-ENOMEM);
  }

  sbd->tune->b_try_leaf_mrg = 
    (sbd->tune->pr.pr_try_leaf_mrg * (blocksize - sizeof(gfs_leaf_t))) / 100;
  sbd->tune->b_do_leaf_mrg =
    (sbd->tune->pr.pr_do_leaf_mrg * (blocksize - sizeof(gfs_leaf_t))) / 100;

  return 0;
}

  
#ifdef DEBUG

int stack[100];
int stack_ct = 0;

char *fn_names[] = {
  "gfs_dirent_first", /* dir.c */
  "gfs_dirent_next",
  "gfs_dirent_del",
  "gfs_dirent_alloc",
  "leaf_search",
  "get_leaf",
  "get_leaf_nr",
  "get_first_leaf",
  "get_next_leaf",
  "linked_leaf_search",
  "leaf_pack",
  "leaf_merge",
  "get_adj_leafsize",
  "dir_halve_exhash",
  "check_leaf_merge",
  "dir_make_exhash",
  "dir_split_leaf",
  "dir_double_exhash",
  "dir_e_search",
  "dir_e_add",
  "dir_e_del",
  "do_filldir",
  "dir_e_trans",
  "dir_e_mvino",
  "dir_l_search",
  "dir_l_add",
  "dir_l_del",
  "dir_l_trans",
  "dir_l_mvino",
  "dir_stats",
  "gfs_dir_search",
  "gfs_dir_add",
  "gfs_dir_del",
  "gfs_dir_trans",
  "gfs_dir_mvino",
  "gfs_dir_free",
  "gfs_ino2gl", /* fs.c */
  "gfs_ihashlookup",
  "gfs_ihashadd",
  "gfs_ihashdel",
  "gfs_indirfree",
  "freefile",
  "gfs_difree",
  "gfs_random",
  "gfs_randrange",
  "gfs_change_nlink",
  "gfs_truncate",
  "gfs_inode_attr_in", /* inode.c */ 
  "gfs_inode_attr_out",
  "gfs_lookupi",
  "gfs_lookup",
  "gfs_createi",
  "gfs_create",
  "gfs_link",
  "unlinki",
  "gfs_unlink",
  "gfs_symlink",
  "gfs_mkdir",
  "rmdiri",
  "gfs_rmdir",
  "gfs_mknod",
  "gfs_is_subdir",
  "gfs_rename",
  "gfs_readlink",
  "gfs_irevalidate",
  "gfs_follow_link",
  "gfs_bmap"
};

#endif

