/******************************************************************************
*
*    Copyright 1999 Regents of the University of Minnesota
*
*    This is free software released under the GNU General Public License.
*    There is no warranty for this software.  See the file COPYING for
*    details.
*
*    See the file CONTRIBUTORS for a list of contributors.
*
*    This file is maintained by:
*      Mike C Tilstra <tilstra@lcse.umn.edu>
* 
*    Module name: dlock_tcp.o
*
* This is an attempt to gather some stats about this lock module and make
* them availible in a proc file entry.
*/

#include <linux/fs.h>
#include <linux/proc_fs.h>

/******************************************************************************
 * These few globals hold the counts for various things
 */
static unsigned long tcpdlocks = 0;
static unsigned long glocks    = 0;
static unsigned long gunlocks  = 0;

/******************************************************************************
 * the count incramenters
 */
void incr_dltcp(void)
{
	tcpdlocks++;
}
void incr_glock(void)
{
	glocks++;
}
void incr_gunlock(void)
{
	gunlocks++;
}

/******************************************************************************
 * dlip_read_proc()
 */
#define PLIMIT (PAGE_SIZE - 80)
int dlip_read_proc(char *buf, char **start, off_t offset,
                   int len, int unused)
{
	len =0;
	len += sprintf(buf+len, "glocks      %lu\n"
			                  "gunlocks    %lu\n"
									"tcp dlocks  %lu\n",
									glocks, gunlocks, tcpdlocks);
					
	if( len > PLIMIT ) return len;

	return len;
}

struct proc_dir_entry dlip_proc_entry = {
        0,
        4, "dlip",
        S_IFREG | S_IRUGO,
        1, 0, 0,
        0, NULL,
        &dlip_read_proc,
};

/******************************************************************************
 * some wrappers for the install & uninstall functions so we can keep
 * things to this file.
 */
void dlip_init_proc(void)
{
	proc_register( &proc_root, &dlip_proc_entry );
}
void dlip_uninit_proc(void)
{
	proc_unregister(&proc_root, dlip_proc_entry.low_ino);
}


