/******************************************************************************
*******************************************************************************
**
**    Copyright 1999 Sistina Software, Inc.  <kpreslan@sistina.com>
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@lcse.umn.edu>
** 
**    Module name: main.c
**
*******************************************************************************
******************************************************************************/

#define __NO_VERSION__ 
#include <linux/module.h>
#include <linux/version.h>
char kernel_version [] = UTS_RELEASE;

#include <linux/kernel.h>
#include <linux/slab.h>

#include "gfs_locking.h"



typedef struct stupid_s
{
  char table_name[GFS_LOCKNAME_LEN + 1];

  gfs_callback_t *cb;
  void *fsdata;

  struct stupid_s *next;
} stupid_t;



stupid_t *list = NULL;



/******************************************************************************
*******************************************************************************
**
** stupid_mount()
**
** Description:
**
** Input(s):
**
** Output(s):
**
** Returns:
**
*******************************************************************************
******************************************************************************/

void *stupid_mount(char *table_name, char *host_data, gfs_callback_t *cb, void *fsdata)
{
  stupid_t *moddata;


  MOD_INC_USE_COUNT;

  printk("Entering stupid_mount(%s, %s)\n", table_name, host_data);


  for (moddata = list; moddata; moddata = moddata->next)
    if (!strcmp(moddata->table_name, table_name))
    {
      printk("Exiting stupid_mount() = table_exists\n");
      MOD_DEC_USE_COUNT;
      return(NULL);
    }


  moddata = (stupid_t *)kmalloc(sizeof(stupid_t), GFP_KERNEL);
  if (!moddata)
  {
    printk("Exiting stupid_mount() = out of memory\n");
    MOD_DEC_USE_COUNT;
    return(NULL);
  }
  memset(moddata, 0, sizeof(stupid_t));


  strcpy(moddata->table_name, table_name);

  moddata->cb = cb;
  moddata->fsdata = fsdata;


  moddata->next = list;
  list = moddata;


  printk("Exiting stupid_mount() = success\n");

  return(moddata);
}


/******************************************************************************
*******************************************************************************
**
** stupid_unmount()
**
** Description:
**
** Input(s):
**
** Output(s):
**
** Returns:
**
*******************************************************************************
******************************************************************************/

void stupid_unmount(void *moddata)
{
  stupid_t *stu = (stupid_t *)moddata;
  stupid_t *tmp, *last = NULL;
  int done = FALSE;


  printk("Entering stupid_unmount(%s)\n", stu->table_name);


  if (list == moddata)
  {
    list = list->next;
    done = TRUE;
  }
  else
    for (tmp = list; tmp; last = tmp, tmp = tmp->next)
      if (tmp == moddata)
      {
	last->next = tmp->next;
	done = TRUE;
      }


  if (!done)
    printk("Given a weird moddata pointer\n");
  else
  {
    kfree(moddata);
    MOD_DEC_USE_COUNT;
  }


  printk("Exiting stupid_unmount()\n");
}


/******************************************************************************
*******************************************************************************
**
** stupid_lock()
**
** Description:
**
** Input(s):
**
** Output(s):
**
** Returns:
**
*******************************************************************************
******************************************************************************/

int stupid_lock(void *moddata, uint64 lock, int op, int flags, void **lockdata)
{
  stupid_t *stu = (stupid_t *)moddata;
  int ret = 0;

  printk("Entering stupid_lock()\n");

  printk("  table = %s, lock = %"FMT_64"u, op = 0x%.8X, flags = 0x%.8X, *lockdata = %s\n",
	 stu->table_name, lock, op, flags, (*lockdata) ? "Not NULL" : "NULL");

  *lockdata = stu;

  ret = (LCKOUT_HELD) | ((flags & LCK_SHARED) ? LCKOUT_SHARED : 0) | (LCKOUT_CACHEABLE) | (LCKOUT_NEED_E);

  printk("  ret = 0x%.8X\n", ret);

  printk("Exiting stupid_lock()\n");

  return(ret);
}


/******************************************************************************
*******************************************************************************
**
** stupid_unlock()
**
** Description:
**
** Input(s):
**
** Output(s):
**
** Returns:
**
*******************************************************************************
******************************************************************************/

int stupid_unlock(void *moddata, uint64 lock, int flags, void **lockdata)
{
  stupid_t *stu = (stupid_t *)moddata;


  printk("Entering stupid_unlock()\n");

  printk("  table = %s, lock = %"FMT_64"u, flags = 0x%.8X, *lockdata = %s\n",
	 stu->table_name, lock, flags, (*lockdata) ? "Not NULL" : "NULL");

  *lockdata = stu;

  printk("Exiting stupid_unlock()\n");


  return(0);
}


/******************************************************************************
*******************************************************************************
**
** stupid_release()
**
** Description:
**
** Input(s):
**
** Output(s):
**
** Returns:
**
*******************************************************************************
******************************************************************************/

void stupid_release(void *moddata, void **lockdata)
{
  stupid_t *stu = (stupid_t *)moddata;
  

  printk("Entering stupid_release(%s)\n", stu->table_name);

  *lockdata = NULL;

  printk("Exiting stupid_release()\n");
}


gfs_lock_proto_t stupid_lock_proto =
{
  "stupidlock",
  stupid_mount,
  stupid_unmount,
  stupid_lock,
  stupid_unlock,
  stupid_release,
  NULL
};


/******************************************************************************
*******************************************************************************
**
** init_module()
**
** Description:
**
** Input(s):
**
** Output(s):
**
** Returns:
**
*******************************************************************************
******************************************************************************/

int init_module(void)
{
  int error;

  printk("Entering init_module()\n");


  if ((error = register_lock_proto(&stupid_lock_proto)) < 0)
    return(error);

  printk("Stupid Lock Protocol Installed\n");


  printk("Exiting init_module()\n");

  return(0);
}


/******************************************************************************
*******************************************************************************
**
** cleanup_module()
**
** Description:
**
** Input(s):
**
** Output(s):
**
** Returns:
**
*******************************************************************************
******************************************************************************/

void cleanup_module(void)
{
  printk("Entering cleanup_module()\n");

  unregister_lock_proto(&stupid_lock_proto);

  printk("Stupid Lock Protocol Uninstalled\n");

  printk("Exiting cleanup_module()\n");
}



/*  Symbols  */
EXPORT_NO_SYMBOLS;


