/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-1999 Regents of the University of Minnesota
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      David C. Teigland  <teigland@lcse.umn.edu>
** 
**    Module name: lock.h
**
**    Description:
**	Device lock header file.
**
*******************************************************************************
******************************************************************************/

#ifndef __LOCK_H__
#define __LOCK_H__


typedef struct dlockdev_s
{
  uint64       	dl_start;	/* starting lock in logical sp array */
  uint32       	dl_nlocks;	/* number of locks on this device */
  kdev_t        dl_dev;		/* device number */
  uint32	dl_adev;	/* user space device number */
  struct inode 	*inode;
  struct file  	*file;
} dlockdev_t;


#if (DLOCK_VERSION == 2)
/* device lock ioctl flags to determine when to give up and how */

#define PL_NDELAY		0x00000001
#define PL_TRY			0x00000002
#define PL_RESET		0x00000004

/* Do an exponential backoff on the attempt instead of continusly 
 * trying the lock */

#define PL_EXP			0x00000008


#define SC_TAG_HEAD		0x21
#define DL_TIMEOUT		60*HZ
#define DL_MAXRETRY		3

#define POOL_RESET_TIMEOUT_US   20000000
#define POOL_CNF_DELAY_US	50000
#define DLOCK_ROLL_TIME 	3600


#define LOCK_OPCODE		0xA0
#define LOCK_CDBLEN		12
#define LOCK_RETSIZE		8

#define LOCK_LOCK               0x01
#define LOCK_UNLOCK             0x02
#define LOCK_UNLOCK_INCR        0x03
#define LOCK_RESET              0x04
#define LOCK_ACTIVITY_ON        0x05
#define LOCK_ACTIVITY_OFF       0x06



typedef struct dlock_s
{
  uint32	l_state:1,     
                l_activity:1,
                l_clock:30;
  time_t	l_ultime;	/* time this lock was last UNLOCKed */
} dlock_t;


typedef struct dlock_req_s 
{
  uint32	l_id;		/* logical lock number */
  uint32	l_flags;       	/* flags */
  uint32	l_counter;	/* dlock clock */
  uint32	l_cacheable;	/* True if the clock hasn't changed */
  uint32      	l_timeout;	/* Timeout reached */
} dlock_req_t;


typedef struct dlock_params_s
{
  dlockdev_t  	*dp_dldev;     	/* device */
  uint32       	dp_spool;      	/* spool this lock maps to */
  uint32       	dp_devnum;     	/* device number */
  uint32       	dp_phyln;      	/* physical lock number */
  uint32       	dp_action;     	/* dlock opertation */
  dlock_req_t   *dp_req;    	/* iotctl arg */
} dlock_params_t;


typedef struct dlock_ret_s
{
  uint32      	r_status;
  uint32      	r_result;
  dlock_t   	r_dlock;
} dlock_ret_t;

#endif


#if (DLOCK_VERSION == 3)

#define SC_TAG_HEAD		0x21
#define DL_TIMEOUT		60*HZ
#define DL_MAXRETRY		3

typedef struct pool_dlock_s
{
  u_char        dl_cdb[16];
  
  int           dl_ln_not_device;

  uint32        dl_device;

  uint32        dl_ln;
  int           dl_lnoffset;
  int           dl_lnsize;

  int           dl_alloc_len;
} pool_dlock_t;


typedef struct dlock_params_s
{
  dlockdev_t   *dp_dldev;
  pool_dlock_t  *dp_dlock;
} dlock_params_t;

#endif


#endif /* __LOCK_H__ */









