/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-1999 Regents of the University of Minnesota
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      David C. Teigland  <teigland@lcse.umn.edu>
** 
**    Module name: pool_linux.h
**
**    Description:
**
*******************************************************************************
******************************************************************************/

#ifndef __POOL_LINUX_H__
#define __POOL_LINUX_H__

#define POOL_MAJOR   		(121)

#define GMAJOR 			MAJOR
#define GMINOR 			MINOR
#define KRET(x)			(-x)
#define KM_SLEEP 		GFP_KERNEL
#define kmem_alloc 		kmalloc
#define kmem_free(pntr, len) 	kfree(pntr)
#define vmem_free(pntr, len) 	vfree(pntr)
#define makedevice 		MKDEV
#define copyin(from, to, size) 	copy_from_user(to, from, size)
#define copyout(from, to, size)	copy_to_user(to, from, size)


#ifndef ASSERT
#define ASSERT(x)  \
  if (!(x)) {	\
    printk("ASSERTION FAILED: %s\n", #x); \
  } 
#endif /* ASSERT */


#ifdef __ANSI_CPP__
#define POOL_SUNLOCK(lockp,flags) \
  {\
     IFDEBUG(POOL_DEBUG_SLOCK, \
	     printk("\t(unlocking %s, file %s, line %d)\n", \
                   #lockp, __FILE__, __LINE__);); \
     spin_unlock_irqrestore(lockp,flags);  \
  }
#else
#define POOL_SUNLOCK(lockp,flags) \
  {\
     IFDEBUG(POOL_DEBUG_SLOCK, \
	     printk("\t(unlocking %s, file %s, line %d)\n", \
		   "lockp", __FILE__, __LINE__);); \
     spin_unlock_irqrestore(lockp,flags);   \
  }
#endif /* ANSI Cpp */

#ifdef __ANSI_CPP__
#define POOL_SLOCK(plret,lockp,flags) \
  {\
     IFDEBUG(POOL_DEBUG_SLOCK, \
	     printk("\t(locking %s, file %s, line %d)\n", #lockp,\
		   __FILE__, __LINE__);); \
     spin_lock_irqsave(lockp, plret);  \
  }	   
#else
#define POOL_SLOCK(plret,lockp,flags) \
  {\
     IFDEBUG(POOL_DEBUG_SLOCK, \
	     printk("\t(locking %s, file %s, line %d)\n", "lockp",\
		   __FILE__, __LINE__);); \
     spin_lock_irqsave(lockp, plret);  \
  }	   
#endif /* ANSI Cpp */




int pool_ioctl(struct inode *inode, struct file *file, unsigned int cmd, 
	       unsigned long arg);
int bdopen(kdev_t dev, struct inode **inodep, struct file **filep);
int bdclose(kdev_t dev, struct inode **inodep, struct file **filep);
unsigned long gencopy(void *to, void *from, unsigned long size);


#endif /* __POOL_LINUX_H__ */

