/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-1999 Regents of the University of Minnesota
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      David C. Teigland  <teigland@lcse.umn.edu>
** 
**    Module name: pool.h
**
**    Description:
**	Pool driver primary header file.
**
*******************************************************************************
******************************************************************************/

#ifndef __POOL_H__
#define __POOL_H__

/* #include "NSP.cf" */
#include "pool_ondisk.h"
#include "pool_incore.h"
#include "pool_common.h"
#include "pool_stats.h"
#include "pool_debug.h"

#ifdef linux
#include "lock_linux.h"
#include "pool_linux.h"
#endif


#define POOL_MAX_DEVICES 	32
#define POOL_MAGIC 		0x011670
#define POOL_VERSION		(NSPMajorVersion << 16 | \
				 NSPMinorVersion <<  8 | \
				 NSPUpdateLevel)
#define BBSHIFT 		9

#define POOL_BUSY(pp) 		((pp)->p_oblk > 0)


/* Pool flags.   These are used in the status field of pool_t structures.
 *
 * PF_INIT:	 Temp state while pool is being added/generated.
 * PF_LOWEROPEN: In-use pool's lower devices are open. Normal state.
 * PF_VALID: 	 State when pool is not being initialized.  It's cleared to
 *		 prevent two adds from interfearing. Complement of PF_INIT.
 * PF_FREE: 	 Pool is not in use and free to be assigned.
 *
 * Unused pools have the state (PF_VALID | PF_FREE). 	  	(flag=6)
 * In-use pools have the state (PF_VALID | PF_LOWEROPEN).	(flag=3)
 * Pools being initialized are (PF_INIT).			(flag=0)
 */

#define	PF_INIT			0x00000000
#define PF_LOWEROPEN		0x00000001
#define PF_VALID		0x00000002
#define	PF_FREE			0x00000004


/* IO Control Commands */

#define _POOLC_(x)	(('p' << 8) | x)

#if (DLOCK_VERSION == 2)
#define POOL_LOCK               _POOLC_(0x00000001)
#define POOL_UNLOCK             _POOLC_(0x00000002)
#define POOL_UNLOCK_INCR        _POOLC_(0x00000003)
#define POOL_RESET              _POOLC_(0x00000004)
#define POOL_ACTIVITY_ON        _POOLC_(0x00000005)
#define POOL_ACTIVITY_OFF       _POOLC_(0x00000006)
#define POOL_CLEAR_ALL_LOCKS    _POOLC_(0x00000009)
#endif

#define POOL_DUMP_TOTALSTATS    _POOLC_(0x00000011)
#define POOL_DUMP_DISKSTATS     _POOLC_(0x00000012)
#define POOL_DUMP_LOCKSTATS     _POOLC_(0x00000013)
#define POOL_CLEAR_STATS        _POOLC_(0x00000014)
#define POOL_READ_CAPACITY      _POOLC_(0x00000015)
#define POOL_DUMP_LOCKTAB       _POOLC_(0x00000016)
#define POOL_DLOCK_COUNT	_POOLC_(0x00000017)

#define POOL_IND_CACHE		_POOLC_(0x00000021)

#define POOL_ADD                _POOLC_(0x00000041)
#define POOL_REMOVE             _POOLC_(0x00000042)
#define POOL_LIST		_POOLC_(0x00000043)
#define POOL_SPLIST		_POOLC_(0x00000044)

#define POOL_DEBUG_ON           _POOLC_(0x00000100)
#define POOL_DEBUG_OFF          _POOLC_(0x00000101)

#define POOL_READCAPACITY	DIOCREADCAPACITY



#endif /* __POOL_H__ */







