/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-1998 Regents of the University of Minnesota
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      David C. Teigland  <teigland@lcse.umn.edu>
** 
**    Module name: pool_head.h
**
**    Description:
**	Network storage pool device driver non-shared header file.
**
*******************************************************************************
******************************************************************************/

/******************************************************************************
*******************************************************************************
**
** $Log: pool_head.h,v $
** Revision 1.1.1.1  1999/07/02 20:41:15  cattelan
** Initial
**
** Revision 1.18  1998/11/18 22:17:05  teigland
** Changed some kmallocs to vmallocs for lots of devices.  Removed old dlock code.
**
** Revision 1.17  1998/10/14 03:34:56  teigland
** Compiles for irix, other stuff.
**
** Revision 1.16  1998/10/06 17:46:17  teigland
** Commented out problem chunk of code in pool_mapping.  Added timestamps to
** dlocks so they won't be cachable after sitting for an hour.
**
** Revision 1.15  1998/10/01 23:22:20  teigland
** Minor changes in mapping related to striping and gencopy.
**
** Revision 1.14  1998/09/23 20:33:20  teigland
** Should fix certain dlock resetting errors.
**
** Revision 1.13  1998/09/22 21:25:25  teigland
** Small changes, mainly debugging messages.
**
** Revision 1.12  1998/09/04 23:03:06  teigland
** Changed makefile and other small changes.
**
** Revision 1.11  1998/09/02 23:35:35  teigland
** Changed maintainer.  Added pool_name field to pool_status, used for listing
** pools.
**
** Revision 1.10  1998/09/01 00:03:40  teigland
** Changed kdev_t's in pool_status_t in major and minor fields to make pool_list
** work properly.  Associated changes in other functions.
**
** Revision 1.9  1998/08/19 23:38:49  teigland
** Added fync in cleanup_module, added FMT_64 in place of ll for alpha, added
** alpha support in Makefile.
**
** Revision 1.8  1998/08/05  19:46:52  teigland
** Added ifdefs and changed "MAJOR" so Irix compiled.  Added gencopy for
** linux and updated ioctl.  Finished off ioctl cases for linux.
**
** Revision 1.7  1998/07/31  23:02:49  cjs
** I don't even remember what I changed.  Let me think...   I added open and 
** close support for sockets.  And I changed the scsi_get_queueable to just 
** use scsi_allocate devices...  Do a diff to figure out what else I did...
**
** Revision 1.6  1998/07/29 16:07:07  cjs
** Move the lower level opens from the upper level opens to assemble time
**
** Revision 1.5  1998/07/27 19:40:48  cjs
** Got pool_linux.c to compile
**
** Revision 1.4  1998/07/27 18:35:30  cjs
** Fixed errors.  Pool.c now compiles under linux
**
** Revision 1.3  1998/07/24 22:54:02  teigland
** Debugging to get pool to compile on IRIX.  Basic function parameter and
** prototype fixes for Linux.
**
** Revision 1.2  1998/07/23  23:31:24  cjs
** Split off the platform dependent and independent code into separate files
**
** Revision 1.1  1998/07/22 18:55:44  cjs
** Initial import of pool sources
**
** Revision 1.7  1998/07/01 19:37:55  kpreslan
** Fixed the weasel words to reflect the fact that GFS is under the GPL.
**
** Revision 1.6  1998/06/26  09:34:18  kpreslan
** Fixed Copyright weasel words.
**
** Revision 1.5  1998/02/27  01:20:48  cjs
** Pool changes to fix performance and add support for IRIX 6.4
**
** Revision 1.4  1998/02/03 01:21:24  cjs
** Major Pool revision.  See comments in pool.c
**
** Revision 1.3  1997/08/22 13:51:27  grant
** Edited the log header.
**
** Revision 1.2  1997/06/24  20:38:54  kpreslan
** Changes to allow dynamic changing of the debug level.
**
** Revision 1.1.1.1  1997/06/02  14:24:45  grant
** Initial import
**
*******************************************************************************
******************************************************************************/

#ifndef __POOL_HEAD_H__
#define __POOL_HEAD_H__

#ifdef __cplusplus
extern "C" {
#endif

#ident "$Revision: 1.1.1.1 $"

#include "global.h"
#ifdef IRIX
#include <sys/scsiInitiator.h>
#endif


/* Preprocessor Defines */

#define POOL_STATS	     1
#define POOL_STATS2	     1
#define POOL_MAX_DEVICES     32


/* Pool flags.  These are used in the status field of pool_t structures */

#define		PF_INIT			0x00000000
#define 	PF_LOWEROPEN		0x00000001
#define 	PF_VALID		0x00000002
#define		PF_FREE			0x00000004
#define		PF_DEALLOC		0x00000008

#ifdef IRIX
#define B_GFS_CACHE     0x00000040     /* device will cache blocks */
#define B_GFS_INVMISS   0x00000080     /* invalidation miss */
#endif
#ifdef linux
#define POOL_RESET_TIMEOUT_US   20000000
#define POOL_CNF_DELAY_US	50000
#endif
#define DLOCK_ROLL_TIME 	3600

/* Function Macros */

#ifdef IRIX
#define TIMER(val)  val = (iotimer_size > 32 ? (*(uint64 *)iotimer_addr) : \
		  (uint64)(*(uint32 *)iotimer_addr))
#define printk printf
#define KRET(x) x
#define vmalloc kmem_alloc
#define vmem_free kmem_free
#define GMAJOR  emajor
#define GMINOR  minor
#define POOL_BUSY(pp) ((pp)->p_ochr > 0 || (pp)->p_oblk > 0 || (pp)->p_olyr > 0)
#define memset(x, y, z) bzero(x, z)
#endif
#ifdef linux
#define GMAJOR MAJOR
#define GMINOR MINOR
#define TIMER(val) do_gettimeofday(&val)
#define KRET(x) -x
#define KM_SLEEP  GFP_KERNEL
#define kmem_alloc kmalloc
#define kmem_free(pntr, len)  kfree(pntr)
#define vmem_free(pntr, len)  vfree(pntr)
#define makedevice  MKDEV
#define copyin(from, to, size) copy_from_user(to, from, size)
#define copyout(from, to, size) copy_to_user(to, from, size)
#define POOL_BUSY(pp) ((pp)->p_oblk > 0)
#define BBSHIFT 9
#ifndef ASSERT
#define ASSERT(x)  \
  if (!(x)) {	\
    printk("ASSERTION FAILED: %s\n", #x); \
  } 
#endif /* ASSERT */
#endif

#define bptoq(bp)       ((pool_queue_t *)((bp)->b_fsprivate))

  /* locking macros */

#ifdef IRIX
#ifdef __ANSI_CPP__
#define POOL_SUNLOCK(lockp,pl) \
  {\
     IFDEBUG(POOL_DEBUG_SLOCK, \
	     printf("\tunlocking %s, file %s, line %d, val: %x, pl: %x\n", \
                   #lockp, __FILE__, __LINE__, *lockp, pl);); \
     UNLOCK(lockp,pl);  \
     IFDEBUG(POOL_DEBUG_SLOCK, \
	     printf("\tunlocked %s, val %x\n", #lockp, *lockp);); \
  }
#else
#define POOL_SUNLOCK(lockp,pl) \
  {\
     IFDEBUG(POOL_DEBUG_SLOCK, \
	     printf("\tunlocking %s, file %s, line %d, val: %x, pl: %x\n", \
		   "lockp", __FILE__, __LINE__, *lockp, pl);); \
     UNLOCK(lockp,pl);   \
     IFDEBUG(POOL_DEBUG_SLOCK, \
	     printf("\tunlocked %s, lock %d\n", "lockp", *lockp);); \
  }
#endif /* ANSI Cpp */

#ifdef __ANSI_CPP__
#define POOL_SLOCK(plret,lockp,plset) \
  {\
     IFDEBUG(POOL_DEBUG_SLOCK, \
	     printf("\tlocking %s, file %s, line %d, val: %x, pl: %x\n", #lockp,\
		   __FILE__, __LINE__, *lockp, plset);); \
     plret = LOCK(lockp,plset);  \
     IFDEBUG(POOL_DEBUG_SLOCK, \
	     printf("\tlocked %s, val %x, old pl %x\n", #lockp, *lockp, \
		   plret);); \
  }	   
#else
#define POOL_SLOCK(plret,lockp,plset) \
  {\
     IFDEBUG(POOL_DEBUG_SLOCK, \
	     printf("\tlocking %s, file %s, line %d, val: %x, pl: %x\n", "lockp",\
		   __FILE__, __LINE__, *lockp, plset);); \
     plret = LOCK(lockp,plset);  \
     IFDEBUG(POOL_DEBUG_SLOCK, \
	     printf("\tlocked %s, val %x, old pl %x\n", #lockp, *lockp, \
		   plret);); \
  }	   
#endif /* ANSI Cpp */
#endif

#ifdef linux
#ifdef __ANSI_CPP__
#define POOL_SUNLOCK(lockp,flags) \
  {\
     IFDEBUG(POOL_DEBUG_SLOCK, \
	     printk("\t(unlocking %s, file %s, line %d)\n", \
                   #lockp, __FILE__, __LINE__);); \
     spin_unlock_irqrestore(lockp,flags);  \
  }
#else
#define POOL_SUNLOCK(lockp,flags) \
  {\
     IFDEBUG(POOL_DEBUG_SLOCK, \
	     printk("\t(unlocking %s, file %s, line %d)\n", \
		   "lockp", __FILE__, __LINE__);); \
     spin_unlock_irqrestore(lockp,flags);   \
  }
#endif /* ANSI Cpp */

#ifdef __ANSI_CPP__
#define POOL_SLOCK(plret,lockp,flags) \
  {\
     IFDEBUG(POOL_DEBUG_SLOCK, \
	     printk("\t(locking %s, file %s, line %d)\n", #lockp,\
		   __FILE__, __LINE__);); \
     spin_lock_irqsave(lockp, plret);  \
  }	   
#else
#define POOL_SLOCK(plret,lockp,flags) \
  {\
     IFDEBUG(POOL_DEBUG_SLOCK, \
	     printk("\t(locking %s, file %s, line %d)\n", "lockp",\
		   __FILE__, __LINE__);); \
     spin_lock_irqsave(lockp, plret);  \
  }	   
#endif /* ANSI Cpp */
#endif /* LINUX */


/* external variables */

#ifdef IRIX
extern void (*pool_fc_open)(void);
extern void (*pool_fc_close)(void);
extern void (*pool_fc_execute_command)(void);
extern void (*pool_fc_get_request)(void);
extern void (*pool_fc_free_request)(void);

extern int  (*psiOpen)(scsiPortID, u_int, struct siEndpoint **);
extern void (*psiClose)(struct siEndpoint *);
extern struct siRequest *(*psiGetRequest)(struct siEndpoint *);
extern void (*psiFreeRequest)(struct siEndpoint *, struct siRequest *);
extern void (*psiExecuteCommand)(struct siRequest *);
#endif

/* Type Definitions */

struct pool_s;
struct locktab_s;
struct pool_queue_s;
struct bufhash_s;

/* UDP dlock completion queue */

#ifdef linux
typedef struct inetqueue_s {
  int			sequence;
  struct semaphore 	sem;
} inetqueue_t;
#endif
  
/* pool_queue_t - Elements of the request queue */

#ifdef IRIX
typedef struct pool_queue_s
{
  lock_t		 q_mutex;
  struct pool_s		*q_pp;		/* ptr to pool struct */
  minor_t		 q_minor;       /* minor of dev for this req */
  buf_t			*q_ubp;		/* upper buffer */
  uint32		 q_lower;	/* # of lower buffers */ 
  uint32		 q_donecnt;	/* # of lower requests done */ 
  buf_t			*q_lbp;		/* lower buffer(s) */
  uint64		 q_start;	/* start time */
} pool_queue_t;
#endif

/* Storage pool structure 
 *	One per device, maintains important info.  The offset of a pool_t
 *	structure into the array is the minor number of the pool block and
 *	character special files.  For example: 
 *   crw-------    1 root     sys       71,  1 Jan 21 16:40 bart
 *   crw-------    1 root     sys       71,  2 Jan 21 16:40 homer
 *	means that the pool_t structure for the bart pool is pools[1] and
 *	the pool_t for homer is pools[2]
 *
 * Locking protocol:
 *	Since the pools[] structure is of a fixed, compile time length there is
 *	no need to syncronize access to the array as a whole.  (This is very
 *	similar to the kernel proc table.)  Syncronization to individual
 *	records is achieved through three constructs.  First: p_mutex is a
 *	spinlock (non-sleeping lock) that once obtained (at software priority
 *	spldisk) gives the holder permission to modify any field provided the
 *	following conditions are met.  Second: the p_oblk, p_olyr, and p_ochr
 *	are reference count fields.  The reference count in lyr is incremented
 *	for every layered open we get from another driver.  ochr and oblk are
 *	boolean values signifying that there is still some process that has the
 *	devices opened as a character or block device, respectively.  If any of
 *	these counts are nonzero none of the static pool information can be
 *	changed (even if p_mutex is held.)  Static pool information basically
 *	includes information such as the number of subpools, the size of the
 *	subpools, the number of dlock devices, the label_device dev_t
 *	structure, the pool label, etc.  In addition, the pool can NOT be
 *	unloaded if the the reference counts are nonzero.  Third: the p_flags
 *	field has bit masks that indicate this pool_t field is in various
 *	states.  The structure can be PF_VALID or not PF_VALID.  If the
 *	structure is not valid it essentially means it is being intialized.
 *	Since intialization requires the driver to sleep it can not hold
 *	p_mutex, so the pool is marked invalid.  Since this happens only at
 *	initialization time, the only situation where we should see a pool_t
 *	without PF_VALID is if two pool_adds try to access the same pool_t
 *	structure. If this is the case the second passemble's call to pool_add
 *	will see the missing PF_VALID and return EBUSY.  The second interesting
 *	flag is PF_FREE.  This indiates whether this pool_t structure actually
 *	represents a pool that is in use.  Empty pool_t's (PF_FREE set) can be
 *	used by pool_add to dynamically create new pools.  pool_remove set's
 *	PF_FREE in the corresponding pool_t upon sucess. 
 */

typedef struct pool_s
{
#ifdef linux
  spinlock_t		 p_mutex;	/* for reference counts and flags */
#endif
#ifdef IRIX
  lock_t		 p_mutex;
#endif
  uint32		 p_oblk;	/* # times opened as block dev */
#ifdef IRIX
  uint32		 p_ochr;	/* # times opened as character dev */
  uint32		 p_olyr;	/* # times opened as layered dev */
#endif
  uint32		 p_flags;	/* status flags */

  /* stats: protected by p_statslock */

#ifdef linux
  spinlock_t		 p_statslock;	/* just for stats */
#endif
#ifdef IRIX
  lock_t		 p_statslock;
#endif
  pool_stat_t		 p_stats;	/* total io stats */
  pool_dstat_t		 p_dstats[POOL_DSTATS_MAX]; /* dev stats */
  pool_dlstat_t		 p_dlstats;	/* device lock stats */

  /* lock table: protected by p_dlmutex */
  
#ifdef linux
  spinlock_t		 p_dlmutex;	/* for lock table struct */
#endif
#ifdef IRIX
  lock_t		 p_dlmutex;
#endif
  struct locktab_s	*p_locktab;	/* table of lock states */

  /* read-only values: not protected, except by POOL_VALID 
   * which prevents opening when the structure is invalid
   * (during pool adds, first open, and final close) */

  kdev_t                 p_label_dev;   /* label device #'s */
  pool_labels_t		 p_label;	/* pool labels */  
  dlockdev_t		*p_dlockdevs;	/* device lock devices */
  subpool_t		*p_subpoollist;	/* subpools */
} pool_t;

/* Device lock parameters */

typedef struct dlock_params_s
{
  dlockdev_t	*dp_dldev;		/* device */
  uint32	 dp_devnum;		/* device number */
  lock_ctl_t	*dp_ctlarg;		/* iotctl arg */
  uint32	 dp_phyln;		/* physical lock number */
  uint32	 dp_action;
} dlock_params_t;


/* Device lock table - One for each device lock */

typedef struct locktab_s
{
  uint32	l_state:1,
    		l_activity:1,
    		l_clock:30;
  time_t	l_ultime;
} locktab_t;


/* Lock command return status */

typedef struct lock_ret_s
{
  uint32      r_status;
  uint32      r_result;
  locktab_t   r_locktab;
} lock_ret_t;


/* Function Prototypes */

int pool_add(pool_add_t *add);
int pool_remove(uint32_t dev);
int pool_list(pool_list_t *list);
int pool_splist(pool_t *pool, pool_splist_t *list);
int pool_mapping(pool_t *pp, uint64 psector, uint32 pbcount, 
		 uint64 *dev_sector, uint32 *dev_bcount, device_t **out_dev);
int lock_mapping(pool_t *pp, dlock_params_t *params); 
int pool_dlock(pool_t *pp, dlock_params_t *params);
time_t timestamp(void);
#ifdef DEBUG
int print_pool(pool_t *pp);
#endif
#ifdef linux
void us_delay(int delay);
int  open_devices(pool_t *pp);
int  close_devices(pool_t *pp);
#endif
int free_pool(pool_t *pool);

#ifdef __cplusplus
}
#endif

#endif /* __POOL_HEAD_H__ */
