/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-1999 Regents of the University of Minnesota
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      David C. Teigland  <teigland@lcse.umn.edu>
** 
**    Module name: pool_stats.h
**
**    Description:
**	Network storage pool device driver shared header file for in-core
**	structures.
**
*******************************************************************************
******************************************************************************/

#ifndef __POOL_STATS_H__
#define __POOL_STATS_H__


#define POOL_STATS	     	1
#define POOL_STATS2	     	1


/* pool_stat_t - Total statistical information */

typedef struct pool_stat_s
{
  uint32      	s_reads;	/* # reads */
  uint32       	s_readblks; 	/* # blocks reads */
  uint64      	s_readtime; 	/* time spent reading (usec) */
  uint32      	s_writes;       /* # writes */
  uint32       	s_writeblks; 	/* # blocks written */
  uint64     	s_writetime; 	/* time spent writing (usec) */
  uint32      	s_invmiss; 	/* # invalidation misses */
  uint64      	s_invtime; 	/* time spent from inv miss (usec) */
} pool_stat_t;

/* pool_dlstat_t - Total device lock statistical information */

typedef struct pool_dlstat_s
{
  uint32     	s_cacheable; 	/* cacheable accesses */
  uint32       	s_locks;       	/* successful locks */
  uint32       	s_conflicts; 	/* lock conflicts */
  uint64     	s_conflict_time;/* time spent on conflicts (usec) */
  uint32      	s_unlocks; 	/* unlocks */
  uint32      	s_unlock_incrs;	/* unlocks with incr */
  uint64       	s_time;		/* time spent on dlocks (usec) */
} pool_dlstat_t;

/* pool_dstat_t - One per device, statistical information */

typedef struct pool_dstat_s
{
  kdev_t       	s_dev;		/* device number */
  int8	     	s_out_reads;	/* outstanding reads */	
  int8	      	s_out_writes;	/* outstanding writes */ 
} pool_dstat_t;


#endif /* __POOL_STATS_H__ */
