/******************************************************************************
*******************************************************************************
**
**    Copyright 1999 Sistina Software, Inc.  <kpreslan@sistina.com>
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      Kenneth W. Preslan <kpreslan@sistina.com>
** 
**    Module name: do_dlock-0.6.c
**
*******************************************************************************
******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "scsi_lib.h"



#define T (1)
#define F (0)





int main(int argc, char *argv[])
{
  int fd;
  unsigned char cdb[12], data[16384];
  unsigned int action, locknumber, inputversionnumber;
  int result;



  if (argc < 5)
  {
    fputs("do_dlock-0.6 DeviceName Action LockNumber InputVersionNumber\n", stderr);
    fputs("\noutput for all actions\n", stderr);
    fputs("  Result State Activity Version\n", stderr);
    exit(EXIT_FAILURE);
  }



  if ((fd = scsi_open(argv[1])) < 0)
  {
    fprintf(stderr, "do_dlock-0.6:  can't open device %s:  %s\n", argv[1], strerror(errno));
    exit(EXIT_FAILURE);
  }

  action = atoi(argv[2]);
  locknumber = atoi(argv[3]);
  inputversionnumber = atoi(argv[4]);

  memset(data, 0, 16384);



  memset(cdb, 0, 12);

  cdb[0] = 0xA0;

  cdb[1] = action & 0x0F;

  cdb[2] = (locknumber >> 24) & 0xFF;
  cdb[3] = (locknumber >> 16) & 0xFF;
  cdb[4] = (locknumber >> 8) & 0xFF;
  cdb[5] = (locknumber >> 0) & 0xFF;

  cdb[6] = (inputversionnumber >> 24) & 0xFF;
  cdb[7] = (inputversionnumber >> 16) & 0xFF;
  cdb[8] = (inputversionnumber >> 8) & 0xFF;
  cdb[9] = (inputversionnumber >> 0) & 0xFF;

  cdb[10] = 0x08;

  cdb[11] = 0x00;

#ifdef DEBUG
  for (x = 0; x < 12; x++)
    fprintf(stderr, "%.2X ", cdb[x]);
  fprintf(stderr, "\n");
#endif  /*  DEBUG  */



  if ((result = scsi_cmd(fd, 12, cdb, 0, NULL, 8, data, 0, 0)) < 0)
  {
    fputs("do_dlock-0.6:  bad dlock command\n", stderr);
    exit(EXIT_FAILURE);
  }



#ifdef DEBUG
  fprintf(stderr, "Dlock ok:  %d\n", result);
  for (x = 0; x < result; x++)
  {
    fprintf(stderr, "%.2X ", data[x]);
    if (x % 16 == 15)
      fprintf(stderr, "\n");
  }
  if (x % 16)
    fprintf(stderr, "\n");
#endif  /*  DEBUG  */

  printf("%u_result ", (data[0] >> 7) & 0x01);
  printf("%u_state ", (data[0] >> 6) & 0x01);
  printf("%u_activity ", (data[0] >> 5) & 0x01);
  printf("%u_version ",
	 (data[1] << 24) |
	 (data[2] << 16) |
	 (data[3] << 8) |
	 (data[4] << 0));
  
  printf("\n");



  scsi_close(fd);

  exit(EXIT_SUCCESS);
}

