/******************************************************************************
*******************************************************************************
**
**    Copyright 1995-1999 Regents of the University of Minnesota
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      mike tilstra <tilstra@lcse.umn.edu>
** 
**    Module name: lockprobe
**
**    Description: get the atatus of a lock from an ipdlock server.
**
*******************************************************************************
******************************************************************************/

/* Simple test client that sits in user space.
 */

#include <stdio.h>
#include <rpc/rpc.h>
#include "nddserv.h"

int main(int argc, char **argv) {
	CLIENT *cl;
	CDBs cdb;
	RDFs *rdf;
	unsigned int lock = 0;

	if( (argc!=3) ) {
		printf("Usage: %s server lock\n", argv[0]);
		exit(1);
	}
	lock = atoi(argv[2]);

	if( !(cl = clnt_create(argv[1], NDD_PROGRAM, NDD_VERSION, "udp"))) {
		clnt_pcreateerror(argv[1]);
		exit(1);
	}

	/* get lock count */
	memset(&cdb, 0, sizeof(CDBs));
	cdb.type = 1;
	cdb.CDBs_u.mode.tag = 0;
	if( (rdf = ndd_dodlock_1(&cdb, cl)) ==NULL) {
		printf("Error with modepage.\n");
		exit(1);
	}

	if( rdf->type != 2 ) {
		printf("Error with rdf type\n");
		exit(1);
	}
	if( lock > rdf->RDFs_u.mpag.num_locks ) {
		printf("Lock out of range, largest lock value is %d\n",
				rdf->RDFs_u.mpag.num_locks);
		exit(1);
	}

	memset(&cdb, 0, sizeof(CDBs));
	cdb.type = 0;
	cdb.CDBs_u.dlock.opcode = 0xA0;
	cdb.CDBs_u.dlock.action = 0;
	cdb.CDBs_u.dlock.lock_number = lock;
	if( (rdf = ndd_dodlock_1(&cdb, cl)) ==NULL) {
		printf("Error with cdb nop.\n");
		exit(1);
	}

	if( rdf->type != 0 ) {
		printf("Error with rdf type\n");
		exit(1);
	}

	printf("rdf-> result    = %d\n"
			 "      act       = %d\n"
			 "      expired   = %d\n"
			 "      state     = %d\n"
			 "      version # = %d\n"
			 "      # holders = %d\n",
			 rdf->RDFs_u.dlock.result,
			 rdf->RDFs_u.dlock.act,
			 rdf->RDFs_u.dlock.expired,
			 rdf->RDFs_u.dlock.state,
			 rdf->RDFs_u.dlock.version_number,
			 rdf->RDFs_u.dlock.num_holders );

	exit(0);
}
