#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>

/*#include "main.h"*/
#include "structs.h"
#include "misc.h"
#include "global.h"
#include "suite_debug.h"
#include "state.h"
#include "wrappers.h"

int atoicheck(char *c, int line, char *file) {
  char *endptr;
  int ret;

  ret = strtoul(c, &endptr, 0);

  if (c == endptr) {
    eprintf(DLF_ERROR, "atoic: bad number '%s', line: %d file: %s\n", c, line, file);
    die(NULL);
  }
  
  return ret;
}


int table_match(name_table_t *table, char *arg) {
  int x;

  ENTER(TABLE_MATCH);

  for (x=0; x < table->nr_names; x++) {
    if (!strcmp(arg, table->names[x])) {
      EXIT(TABLE_MATCH);
      return x;
    }
  }

  EXIT(TABLE_MATCH);
  dprintf("table_match: no match found for %s\n", arg);
  return -1;
}

int table_match_part(name_table_t *table, char *arg) {
  int x;

  ENTER(TABLE_MATCH_PART);

  for (x=0; x < table->nr_names; x++) {
    if (!strncmp(arg, table->names[x], strlen(table->names[x]))) {
      EXIT(TABLE_MATCH_PART);
      return x;
    }
  }

  EXIT(TABLE_MATCH_PART);
  return -1;
}

int table_val(name_table_t *table, char *arg, int val) {
  int retval;
  ENTER(TABLE_VAL);

  if (*arg != '-') {

    if (isalpha(*arg)) 
      retval = table_match(table, arg);
    else
      retval = atoic(arg);

  }

  else {retval = val;}
  
  /**val = retval;*/

  EXIT(TABLE_VAL);
  return retval;
}


/* assumes string name is longer than numerical one */

void syms_to_nums(name_table_t *table, char *arg) {
  int index;
  char *tmp = arg;
  char buf[ARG_SIZE];

  ENTER(SYMS_TO_NUMS);

  while (*tmp) {
    if ((index = table_match_part(table, tmp)) != -1 ) {
      char *tmp2;

      tmp2 = &tmp[strlen(table->names[index])];
      *tmp  = '\0';
      sprintf(buf, "%s%d%s", arg, index, tmp2);
      strcpy(arg, buf);
      tmp = arg;
    }
    
    tmp++;
  }

  EXIT(SYMS_TO_NUMS);
}

void read_number_range(number_range_t *range, char *c) {
  char *tmp;

  ENTER(READ_NUMBER_RANGE);

  if ((tmp = strstr(c, "-")) == NULL) {
    range->base = atoic(c);
    range->range = 0;
    range->how   = FIXED;
  }
  else {
    if (*c == '[') {
      range->how = RANDOM;
      range->base = atoic(c+1);
    }

    else {
      range->how = LINEAR;
      range->base = atoic(c);
    }
    
    range->range = atoic(tmp+1) - range->base;
  }

  ENTER(READ_NUMBER_RANGE);
}

void die(char *c) {
  

  fprintf(stderr, c);

  /*printf("%s", buf);*/
  exit(1);
}


int max_holders(device_handle_t *handle, uint32 lock) {
  dlock_state_t st;
  int x, y, z;
  dlock_data_t data;

  memset(&st, 0, sizeof(st));
  st.state = UNLOCKED;

  if (goto_state(handle, &st, lock, 0))
    die("max_holders: can't goto unlocked state\n");

  for (x=0; ; x++) {
    
    data.action = LOCK_SHARED;
    data.locknum = lock;
    data.clientid = x;
    
    if (do_dlock(handle, &data))
      die("max_holders: do_dlock failed\n");

    if (!data.result)
      break;
  }

  if (data.holders != x)
    die("max_holders: something's screwed up\n");

  for (y=0; y < x; y++) {
    for (z = 0; z < data.holders; z++) {
      if (data.clientids[z] == y)
	break;
    }

    if (z == data.holders) 
      die("max_holders: client not found in list\n");

  }

  return data.holders;
}

	  
